# Variables
MODNAME=`grep_prop name $TMPDIR/module.prop`
MODVER=`grep_prop version $TMPDIR/module.prop`
DV=`grep_prop author $TMPDIR/module.prop`
Device=`getprop ro.product.device`
Model=`getprop ro.product.model`
Brand=`getprop ro.product.brand` 
Architecture=`getprop ro.product.cpu.abi`
SDK=`getprop ro.system.build.version.sdk`
Android=`getprop ro.system.build.version.release`
Type=`getprop ro.system.build.type`
Built=`getprop ro.system.build.date`
Time=$(date "+%d, %b - %H:%M %Z")

# Display UI Messages
DEKHO () {
  local sandesh="$1"
  local samay="${2:-0.5}"
  local prakar="$3"
  local rekha=$(( ${#sandesh} + 3 ))
  [ "$rekha" -gt "55" ] && rekha=55
  if [ "$prakar" == "sar" ]; then
    echo ""
    printf '%*s\n' "$rekha" | tr ' ' '-'
    echo " $sandesh"
    printf '%*s\n' "$rekha" | tr ' ' '-'
    echo ""
  else
    echo -e "$sandesh"
  fi 
  sleep "$samay"
}

# Function to Display PowerSaver Toast Mesages
MEOW() {
  am start -a android.intent.action.MAIN -e powersaver "$@" -n powersaver.pro/.MainActivity >> /dev/null
  sleep 2
}

# Check for volume key
CheckKey() {
while true; do
  key_code=$(getevent -qlc 1 | grep "KEY_" | awk '{print $3}')
  if [ -n "$key_code" ]; then
    echo "$key_code"
    sleep 1
    break
  fi
  sleep 0.1
done
}

# Handle Options based on key pressed
OPT() {
while true; do
  key=$(CheckKey)
  case $key in
    KEY_VOLUMEDOWN)
      return 0
      ;;
    KEY_VOLUMEUP)
      return 1
      ;;
    *)
      DEKHO "❌ Invalid Key! Try Again. Key pressed: $key"
      ;;
  esac
done
}

# Verify ZIP
unzip -o "$ZIPFILE" 'verify.sh' -d "$TMPDIR" >&2
if [ ! -f "$TMPDIR/verify.sh" ]; then
DEKHO "- Module files are corrupted, please re-download" 1 "sar"
exit 1
fi
DEKHO "Verifying Module Files..." 1 "sar"
. "$TMPDIR/verify.sh"
extract "$ZIPFILE" 'customize.sh'  "$TMPDIR/.vunzip"
export ZIPFILE
ui_print "- Files verified successfully!"

# Installation Begins
DEKHO "Fetching module info..." 0.5 "sar"
DEKHO "- Author: $DV"
DEKHO "- Module：$MODNAME"
DEKHO "- Version：$MODVER"
echo -e "- Provider：\c"
if [ "$BOOTMODE" ] && [ "$KSU" ]; then
ui_print "🔥KernelSU app"
sed -i '/^description=/c\description=[🦄 KernelSU Mode] Nukes cache, Disables 200+ GMS services, Disables 95% logs, Applies Kernel Tweaks for Battery and much more. Run Action button to Toggle GMS Services and Cleanup.' "$MODPATH/module.prop"
ui_print "- KernelSU：$KSU_KERNEL_VER_CODE (kernel) + $KSU_VER_CODE (ksud)"
if [ "$(which magisk)" ]; then
ui_print "-----------------------------------------------------------"
ui_print "! Multiple root implementation is NOT supported!"
abort    "-----------------------------------------------------------"
fi
elif [ "$BOOTMODE" ] && [ "$MAGISK_VER_CODE" ]; then
ui_print "💀Magisk app"
sed -i '/^description=/c\description=[🪷 Magisk Mode] Nukes cache, Disables 200+ GMS services, Disables 95% logs, Applies Kernel Tweaks for Battery and much more. Run Action button to Toggle GMS Services and Cleanup.' "$MODPATH/module.prop"
else
ui_print "--------------------------------------------------------"
ui_print "Installation from recovery is not supported"
ui_print "Please install from KernelSU or Magisk app"
abort "----------------------------------------------------------"
fi
## Install App and Verify it installed or not
while true; do
  pm install "$MODPATH/system/product/app/PowerSaverPro/PowerSaverPro.apk" >> /dev/null
  if pm list packages | grep -q "powersaver.pro"; then
      MEOW "Meow!! I will give Toast Messages now"
      break
  else
     DEKHO "❌ Please Allow installation of PowerSaver Assistant"
  fi
done
DEKHO "Fetching device info..." 0.3 "sar"
DEKHO "- Brand：$Brand"
DEKHO "- Device：$Device"
DEKHO "- Model：$Model"
DEKHO "- RAM：$(free | grep Mem | awk '{print $2}')"
DEKHO "Fetching ROM info..." 0.3 "sar"
DEKHO "- Android：$Android"
DEKHO "- Kernel: $(uname -r)"
DEKHO "- CPU：$Architecture"
DEKHO "- SDK：$SDK"
DEKHO "- Build Date：$Built"
DEKHO "- Build Type：$Type"
ui_print "---------------------------------------------"
ui_print "- Running $MODNAME"
ui_print "- Flashing started at $Time "
ui_print "---------------------------------------------"
ui_print " ⚡"
ui_print " 🔋 "
sleep 1
# Print Preview GMS results (will be applied after reboot)
DEKHO "KILLING GMS SERVICES" 0.3 "sar"
DEKHO "✦ Disabled advertising and tracking in Google Play services."
DEKHO "✦ Restricted Google’s data collection on your device."
DEKHO "✦ Disabled the battery-draining HardwareArProviderService."
DEKHO "✦ Disabled bug reporting in Google Play services."
DEKHO "✦ Disabled Google Cast services."
DEKHO "✦ Disabled debugging services in Google Play Services."
DEKHO "✦ Disabled component discovery services in Google Play and Firebase."
DEKHO "✦ Disabled location and time zone services."
DEKHO "✦ Disabled key authentication services in Google Play."
DEKHO "✦ Disabled various background update services."
DEKHO "✦ Disabled services for smartwatches and wearables."
DEKHO "✦ Disabled Trusted Agents / Find My Device services."
DEKHO "✦ Disabled enpromo-related services."
DEKHO "✦ Disabled emergency features and child safety services."
DEKHO "✦ Disabled Google Fit health tracking services."
DEKHO "✦ Disabled Google Nearby services."
DEKHO "✦ Disabled logging and data collection services."
DEKHO "✦ Disabled security and app verification services."
DEKHO "✦ Disabled Google Pay and Wallet services."
DEKHO "✦ Disabled location services."
DEKHO "✦ Disabled Google Play Games-related services."
DEKHO "✦ Disabled Google Instant Apps services."

# Remove modules that conflicts with power saver (if available)
rm -rf /data/adb/modules/AdvanceCleaner
rm -rf /data/adb/modules/Energy_Saver
sleep 1

# Print Preview System Prop Results
DEKHO "DISABLING LOGS & COMPONENTS" 0.1 "sar"
DEKHO "✦ Disabled Real-Time Transport Protocol (RTP) logging"
DEKHO "✦ Disabled JNI (Java Native Interface) checks, reducing logging related to native code interaction."
DEKHO "✦ Stopped logging slow database queries."
DEKHO "✦ Disabled debugging logs related to Qualcomm's Sensor Network Services (SNS)."
DEKHO "✦ Disabled profiling logs for the EGL graphics API."
DEKHO "✦ Disabled logging related to gamed and Wayland."
DEKHO "✦ Disabled logging related to the Hardware Composer (HWC) and SurfaceFlinger components."
DEKHO "✦ Disabled logging related to SurfaceFlinger's backpressure mechanism."
DEKHO "✦ Disabled logging related to surface re-computations."
DEKHO "✦ Disabled logging of Application Not Responding (ANR) events."
DEKHO "✦ Disabled GPU-based pixel buffers, impacting rendering."
DEKHO "✦ Disabled real-time logcat output and streaming."
DEKHO "✦ Disabled TCP metrics logging."
DEKHO "✦ Disabled logging related to the Stagefright media framework."
DEKHO "✦ Disabled various profiling tools."
DEKHO "✦ Disabled StrictMode, which checks for potential performance and security issues."
DEKHO "✦ Changed behavior related to EGL context destruction."
DEKHO "✦ Disabled support for Kernel Samepage Merging (KSM), a memory optimization technique."
DEKHO "✦ Disabled \"checkin\" logging that sends system information to Google."
DEKHO "✦ Disabled debugging capabilities for the device."
DEKHO "✦ Setting the log buffer size to zero, effectively disable the log system."
DEKHO "✦ Disabled the default compression cache."
DEKHO "✦ Disabled JNI checks and QEMU-related logging."
DEKHO "✦ Disabled battery-related logging."
DEKHO "✦ Disabled multiple simultaneous call ringtones."
DEKHO "✦ Disabled the RW logger, used for debugging network interactions."
DEKHO "✦ Disabled IMS (IP Multimedia Subsystem) debug logs."
DEKHO "✦ Disabled debugging related to Wi-Fi Direct (WFD)."
DEKHO "✦ Disabled AAC support for Bluetooth A2DP."
DEKHO "✦ Disabled QMI (Qualcomm MSM Interface) logging for ADB."
DEKHO "✦ Disabled debug logging for the sensors HAL (Hardware Abstraction Layer)."
DEKHO "✦ Disabled logging and crash reports related to Broadcom components."
DEKHO "✦ Disabled an override for USB tethering."
DEKHO "✦ Disabled an OEM socket for the radio."
DEKHO "✦ Disabled virtual WFD capabilities."
DEKHO "✦ Disabled various IMS logging and functionality."
DEKHO "✦ Enabled offline logging for the kernel and logcat."
DEKHO "✦ Disabled OEM-specific dumps"
DEKHO "✦ Enabled the system's sleep mode."
DEKHO "✦ Force disabled error reports and ulog."
DEKHO "✦ Disabled debugging related to display management."
DEKHO "✦ Enabled profiling for game tools."
DEKHO "✦ Disabled GPU performance mode."
DEKHO "✦ Disabled concurrent A2DP connections for FM radio."
DEKHO "✦ Disabled B-frame encoding for the video decoder."
DEKHO "✦ Disabled video decoder debugging."
DEKHO "✦ Disabled Ultra Buffer With Cache (UBWC) for video."
DEKHO "✦ Setting the systems composition type to GPU-based."
DEKHO "✦ Disabled various SurfaceFlinger display options."
DEKHO "✦ Disabled specific tags for the \"atrace\" profiling tool."
DEKHO "✦ Disabled a mechanism for killing tasks that are allocating memory excessively."
DEKHO "✦ Disabled logging for specific services."
DEKHO "✦ Disabled redirecting standard input/output to the log system."
DEKHO "✦ Disabled additional profiling features."
DEKHO "✦ Disabled StrictMode, which checks for potential performance and security issues."
DEKHO "✦ Disabled bytecode verification for the Dalvik VM, impacting security."
DEKHO "✦ Disabled debug logging for memory allocation."
DEKHO "✦ Disabled persistent caching for the A/V system."
DEKHO "✦ Disabled RAM dumps for the SSR (System Server Recovery) service."
DEKHO "✦ Disabled various log outputs related to the systems core functionality, including media playback, rendering, and audio output."
DEKHO "✦ Disabled debug-specific logging and testing"
DEKHO "✦ Disabled Memory Dump"
DEKHO "✦ Disabled CRC"
DEKHO "✦ Disabled IOSTATS"
DEKHO "✦ Disabled Tombstoned"
DEKHO "✦ Disabled Low Memory Killer"
DEKHO "✦ Disabled Dalvik"
DEKHO "✦ Disabled Blur"
DEKHO "✦ Disabled Timer Migration"
echo 0 > /proc/sys/kernel/timer_migration

# Stop/Delete Logs & Cache
DEKHO "DISABLING KERNEL LOGS" 1 "sar"
sysctl -w kernel.panic=0
sysctl -w vm.panic_on_oom=0
sysctl -w kernel.panic_on_oops=0
sysctl -w vm.oom_dump_tasks=0
sysctl -w vm.oom_kill_allocating_task=0
echo "0 0 0 0" > /proc/sys/kernel/printk
echo "off" > /proc/sys/kernel/printk_devkmsg
echo "0" > /proc/sys/debug/exception-trace

DEKHO "OPTIMIZING STORAGE PERFORMANCE" 0.5 "sar"
fstrim -v /cache
fstrim -v /system
fstrim -v /vendor
fstrim -v /data
fstrim -v /persist

DEKHO "APPLYING KERNEL LEVEL TWEAKS FOR BATTERY BACKUP" 1 "sar"

# Action Button
setprop mona-install rukho
cat > "$MODPATH/action.sh" << 'EOF'
#!/bin/sh
MODDIR="${0%/*}"
chmod +x "$MODDIR/mona.sh"
. "$MODDIR/mona.sh"
EOF

# Log script
touch "/data/local/tmp/DisabledAllGoogleServices"
cat > "$MODPATH/mona.sh" << 'EOF'
#!/system/bin/sh
# Log file path
log_file="/sdcard/𝗠𝗘𝗢𝗪𝗻𝗮💅.log"

# Function to write to the log file
log() {
  timestamp=$(date +"%Y-%m-%d %H:%M:%S")
  echo "$timestamp: $1" >> "$log_file" 2>&1
}

# Create log file if it don't exist.
if [ ! -f $log_file ]; then
  touch $log_file
fi

# Functions
# Display UI Messages
DEKHO () {
  local sandesh="$1"
  local samay="${2:-0.5}"
  local prakar="$3"
  local rekha=$(( ${#sandesh} + 3 ))
  [ "$rekha" -gt "55" ] && rekha=55
  if [ "$prakar" == "sar" ]; then
    echo ""
    printf '%*s\n' "$rekha" | tr ' ' '-'
    echo " $sandesh"
    printf '%*s\n' "$rekha" | tr ' ' '-'
    echo ""
  else
    echo -e "$sandesh"
  fi 
  sleep "$samay"
}

# Function to Display PowerSaver Toast Mesages
MEOW() {
  am start -a android.intent.action.MAIN -e powersaver "$@" -n powersaver.pro/.MainActivity >> /dev/null
  sleep 2
}

# Check for volume key
CheckKey() {
while true; do
  key_code=$(getevent -qlc 1 | grep "KEY_" | awk '{print $3}')
  if [ -n "$key_code" ]; then
    echo "$key_code"
    sleep 1
    break
  fi
  sleep 0.1
done
}

# Handle Options based on key pressed
OPT() {
while true; do
  key=$(CheckKey)
  case $key in
    KEY_VOLUMEDOWN)
      return 0
      ;;
    KEY_VOLUMEUP)
      return 1
      ;;
    *)
      DEKHO "❌ Invalid Key! Try Again. Key pressed: $key"
      ;;
  esac
done
}

# Wait for sometimes as before toggling services and cleaning it again
if [ "$(getprop mona-action)" == "khatam" ]; then
echo "----------------------------------------------------------"
echo "❌ Wait for few moments, don't spam like a Retard😭"
echo "----------------------------------------------------------"
MEOW "RUKJA BSDK 😂 CHUTIYA HAI KYA"
sleep 3
exit
fi

# Toggle Services
DEKHO "❔ Which GMS Disabling mode you want to use?" 1 "sar"
DEKHO "🔶 Aggressive Mode\n📝 This may break some features depends on Google\n🔋 Best Mode for Battery Backup\n"
DEKHO "🔷 Safe Mode\n📝 This will not break any features depends on Google\n🔋 Decent Mode for Battery Backup\n"
DEKHO "🔊 Vol+ = 🔶 Aggressive Mode\n🔉 Vol- = 🔷 Safe Mode\n"
OPT
if [ $? -eq 1 ]; then
  DEKHO "✦ All Unnecessary Google Services has been Disabled"
  MEOW "ALL GMS SHIT HAS BEEN FREEZED 🥶"
  ALL="pm disable"
else
  DEKHO "✦ Some Unnecessary Google Services has been Disabled"
  MEOW "SOME GMS SHIT HAS BEEN ENABLED 🥵"
  ALL="pm enable"
fi
DEKHO "\nℹ️ You can change the Mode by Running Action Button later" 1

# Nuke Logs & Cache
log " ⚡ Switching GMS mode × Starting CACHE and LOG deletion "
DEKHO "👾 STARTING CLEANING & TOGGLING GMS" 0.3 "sar"

echo "✦ Deleting Package Cache"
rm -rf /data/package_cache || log "Error deleting /data/package_cache: $?"
sleep 0.3

echo "✦ Deleting Dalvik Cache"
rm -rf /data/dalvik-cache || log "Error deleting /data/dalvik-cache: $?"
sleep 0.3

echo "✦ Deleting Cached Data"
rm -rf /data/cache || log "Error deleting /data/cache: $?"
sleep 0.3

echo "✦ Deleting WLAN Logs"
rm -rf /data/vendor/wlan_logs || log "Error deleting /data/vendor/wlan_logs: $?"
sleep 0.3

echo "✦ Deleting System Logs"
rm -rf /dev/log/* || log "Error deleting /dev/log/*: $?"
sleep 0.3

echo "✦ Deleting System Package Cache"
rm -rf /data/system/package_cache || log "Error deleting /data/system/package_cache: $?"
sleep 0.3

echo "✦ Deleting Thumbnail Cache"
rm -rf /data/media/0/DCIM/.thumbnails || log "Error deleting /data/media/0/DCIM/.thumbnails: $?"
rm -rf /data/media/0/Pictures/.thumbnails || log "Error deleting /data/media/0/Pictures/.thumbnails: $?"
rm -rf /data/media/0/Music/.thumbnails || log "Error deleting /data/media/0/Music/.thumbnails: $?"
rm -rf /data/media/0/Movies/.thumbnails || log "Error deleting /data/media/0/Movies/.thumbnails: $?"
sleep 0.3

echo "✦ Deleting Thermal Logs"
rm -rf /data/vendor/thermal/thermal.dump || log "Error deleting /data/vendor/thermal/thermal.dump: $?"
rm -rf /data/vendor/thermal/last_thermal.dump || log "Error deleting /data/vendor/thermal/last_thermal.dump: $?"
rm -rf /data/vendor/thermal/thermal_history.dump || log "Error deleting /data/vendor/thermal/thermal_history.dump: $?"
rm -rf /data/vendor/thermal/thermal_history_last.dump || log "Error deleting /data/vendor/thermal/thermal_history_last.dump: $?"
sleep 0.3

echo "✦ Deleting ANR Logs"
rm -rf /data/anr/* || log "Error deleting /data/anr/*: $?"
sleep 0.3

echo "✦ Deleting Dev Logs"
rm -rf /dev/log/* || log "Error deleting /dev/log/*: $?"
sleep 0.3

(
until [ "$(getprop mona-action)" == "khatam" -o "$(getprop mona-install)" == "khatam" ]; do
  sleep 1
done
# All Google Services (may break features)
# Toggle the advertising and tracking capabilities of Google Play Services.
$ALL "com.google.android.gms/com.google.android.gms.ads.identifier.service.AdvertisingIdNotificationService"
$ALL "com.google.android.gms/com.google.android.gms.ads.identifier.service.AdvertisingIdService"
$ALL "com.google.android.gms/com.google.android.gms.nearby.mediums.nearfieldcommunication.NfcAdvertisingService"
 
# Restrict/Allow Google's data collection and analytics on your Android device.
$ALL "com.google.android.gms/com.google.android.gms.analytics.AnalyticsService"
$ALL "com.google.android.gms/com.google.android.gms.analytics.AnalyticsTaskService"
$ALL "com.google.android.gms/com.google.android.gms.analytics.internal.PlayLogReportingService"
$ALL "com.google.android.gms/com.google.android.gms.stats.eastworld.EastworldService"
$ALL "com.google.android.gms/com.google.android.gms.stats.service.DropBoxEntryAddedService"
$ALL "com.google.android.gms/com.google.android.gms.stats.PlatformStatsCollectorService"
$ALL "com.google.android.gms/com.google.android.gms.common.stats.GmsCoreStatsService"
$ALL "com.google.android.gms/com.google.android.gms.common.stats.StatsUploadService"
$ALL "com.google.android.gms/com.google.android.gms.backup.stats.BackupStatsService"
$ALL "com.google.android.gms/com.google.android.gms.checkin.CheckinApiService"
$ALL "com.google.android.gms/com.google.android.gms.checkin.CheckinService"
$ALL "com.google.android.gms/com.google.android.gms.tron.CollectionService"
$ALL "com.google.android.gms/com.google.android.gms.common.config.PhenotypeCheckinService"

# Toggle HardwareArProviderService that eats your battery 24×7
$ALL "com.google.android.gms/com.google.android.location.internal.server.HardwareArProviderService"

# Toggle bug reporting functionality of gms.
$ALL "com.google.android.gms/com.google.android.gms.feedback.FeedbackAsyncService"
$ALL "com.google.android.gms/com.google.android.gms.feedback.LegacyBugReportService"
$ALL "com.google.android.gms/com.google.android.gms.feedback.OfflineReportSendTaskService"
$ALL "com.google.android.gms/com.google.android.gms.googlehelp.metrics.ReportBatchedMetricsGcmTaskService"
$ALL "com.google.android.gms/com.google.android.gms.analytics.internal.PlayLogReportingService"
$ALL "com.google.android.gms/com.google.android.gms.locationsharingreporter.service.reporting.periodic.PeriodicReporterMonitoringService"
$ALL "com.google.android.gms/com.google.android.location.reporting.service.ReportingAndroidService"
$ALL "com.google.android.gms/com.google.android.location.reporting.service.ReportingSyncService"
$ALL "com.google.android.gms/com.google.android.gms.common.stats.net.NetworkReportService"
$ALL "com.google.android.gms/com.google.android.gms.presencemanager.service.PresenceManagerPresenceReportService"
$ALL "com.google.android.gms/com.google.android.gms.usagereporting.service.UsageReportingIntentService"

# Toggle Google Cast services.
$ALL "com.google.android.gms/com.google.android.gms.cast.media.CastMediaRoute2ProviderService"
$ALL "com.google.android.gms/com.google.android.gms.cast.media.CastMediaRoute2ProviderService_Isolated"
$ALL "com.google.android.gms/com.google.android.gms.cast.media.CastMediaRoute2ProviderService_Persistent"
$ALL "com.google.android.gms/com.google.android.gms.cast.media.CastMediaRouteProviderService"
$ALL "com.google.android.gms/com.google.android.gms.cast.media.CastMediaRouteProviderService_Isolated"
$ALL "com.google.android.gms/com.google.android.gms.cast.media.CastMediaRouteProviderService_Persistent"
$ALL "com.google.android.gms/com.google.android.gms.cast.media.CastRemoteDisplayProviderService"
$ALL "com.google.android.gms/com.google.android.gms.cast.media.CastRemoteDisplayProviderService_Isolated"
$ALL "com.google.android.gms/com.google.android.gms.cast.media.CastRemoteDisplayProviderService_Persistent"
$ALL "com.google.android.gms/com.google.android.gms.cast.service.CastPersistentService_Persistent"
$ALL "com.google.android.gms/com.google.android.gms.cast.service.CastSocketMultiplexerLifeCycleService"
$ALL "com.google.android.gms/com.google.android.gms.cast.service.CastSocketMultiplexerLifeCycleService_Isolated"
$ALL "com.google.android.gms/com.google.android.gms.cast.service.CastSocketMultiplexerLifeCycleService_Persistent"
$ALL "com.google.android.gms/com.google.android.gms.chimera.CastPersistentBoundBrokerService"

# Toggle debugging services related to Google Play Services.
$ALL "com.google.android.gms/com.google.android.gms.nearby.messages.debug.DebugPokeService"
$ALL "com.google.android.gms/com.google.android.gms.clearcut.debug.ClearcutDebugDumpService"

# Toggle services related to component discovery within Google Play Services and Firebase.
$ALL "com.google.firebase.components.ComponentDiscoveryService"
$ALL "com.google.android.gms/com.google.android.gms.nearby.discovery.service.DiscoveryService"
$ALL "com.google.android.gms/com.google.mlkit.common.internal.MlKitComponentDiscoveryService"

# Toggle services related to location and time zone information.
$ALL "com.google.android.gms/com.google.android.gms.geotimezone.GeoTimeZoneService"
$ALL "com.google.android.gms/com.google.android.gms.location.geocode.GeocodeService"

# Toggle specific services related to Google Play Services, particularly those associated with authentication and account management.
$ALL "com.google.android.gms/com.google.android.gms.chimera.GmsIntentOperationService_AuthAccountIsolated"
$ALL "com.google.android.gms/com.google.android.gms.chimera.GmsIntentOperationService_AuthAccountIsolate"
$ALL "com.google.android.gms/com.google.android.gms.chimera.PersistentApiService_AuthAccountIsolated"
$ALL "com.google.android.gms/com.google.android.gms.chimera.PersistentIntentOperationService_AuthAccountIsolated"

# Toggle various background update services.
$ALL "com.google.android.gms/com.google.android.gms.auth.folsom.service.FolsomPublicKeyUpdateService"
$ALL "com.google.android.gms/com.google.android.gms.fonts.update.UpdateSchedulerService"
$ALL "com.google.android.gms/com.google.android.gms.icing.proxy.IcingInternalCorporaUpdateService"
$ALL "com.google.android.gms/com.google.android.gms.instantapps.routing.DomainFilterUpdateService"
$ALL "com.google.android.gms/com.google.android.gms.mobiledataplan.service.PeriodicUpdaterService"
$ALL "com.google.android.gms/com.google.android.gms.phenotype.service.sync.PackageUpdateTaskService"
$ALL "com.google.android.gms/com.google.android.gms.update.SystemUpdateGcmTaskService"
$ALL "com.google.android.gms/com.google.android.gms.update.SystemUpdateService"
$ALL "com.google.android.gms/com.google.android.gms.update.UpdateFromSdCardService"

# Toggle services related to smartwatches and wearables.
$ALL "com.google.android.gms/com.google.android.gms.backup.wear.BackupSettingsListenerService"
$ALL "com.google.android.gms/com.google.android.gms.dck.service.DckWearableListenerService"
$ALL "com.google.android.gms/com.google.android.gms.fitness.service.wearable.WearableSyncAccountService"
$ALL "com.google.android.gms/com.google.android.gms.fitness.service.wearable.WearableSyncConfigService"
$ALL "com.google.android.gms/com.google.android.gms.fitness.service.wearable.WearableSyncConnectionService"
$ALL "com.google.android.gms/com.google.android.gms.fitness.service.wearable.WearableSyncMessageService"
$ALL "com.google.android.gms/com.google.android.gms.fitness.wearables.WearableSyncService"
$ALL "com.google.android.gms/com.google.android.gms.wearable.service.WearableControlService"
$ALL "com.google.android.gms/com.google.android.gms.wearable.service.WearableService"
$ALL "com.google.android.gms/com.google.android.gms.nearby.fastpair.service.WearableDataListenerService"
$ALL "com.google.android.gms/com.google.android.location.wearable.WearableLocationService"
$ALL "com.google.android.gms/com.google.android.location.fused.wearable.GmsWearableListenerService"
$ALL "com.google.android.gms/com.google.android.gms.mdm.services.MdmPhoneWearableListenerService"
$ALL "com.google.android.gms/com.google.android.gms.tapandpay.wear.WearProxyService"

# Toggle services related to Trusted Agents / Find My Device
$ALL "com.google.android.gms/com.google.android.gms.auth.trustagent.GoogleTrustAgent"
$ALL "com.google.android.gms/com.google.android.gms.trustagent.api.bridge.TrustAgentBridgeService"
$ALL "com.google.android.gms/com.google.android.gms.trustagent.api.state.TrustAgentState"

# Toggle services related to enpromo"
$ALL "com.google.android.gms/com.google.android.gms.enpromo.PromoInternalPersistentService"
$ALL "com.google.android.gms/com.google.android.gms.enpromo.PromoInternalService"

# Toggle services related to emergency features and potentially child safety settings. 
$ALL "com.google.android.gms/com.google.android.gms.thunderbird.EmergencyLocationService" 
$ALL "com.google.android.gms/com.google.android.gms.thunderbird.EmergencyPersistentService"
$ALL "com.google.android.gms/com.google.android.gms.kids.chimera.KidsServiceProxy"
$ALL "com.google.android.gms/com.google.android.gms.personalsafety.service.PersonalSafetyService"

# Toggle services related to Google Fit health and fitness tracking app.
$ALL "com.google.android.gms/com.google.android.gms.fitness.cache.DataUpdateListenerCacheService"
$ALL "com.google.android.gms/com.google.android.gms.fitness.service.ble.FitBleBroker"
$ALL "com.google.android.gms/com.google.android.gms.fitness.service.config.FitConfigBroker"
$ALL "com.google.android.gms/com.google.android.gms.fitness.service.goals.FitGoalsBroker"
$ALL "com.google.android.gms/com.google.android.gms.fitness.service.history.FitHistoryBroker"
$ALL "com.google.android.gms/com.google.android.gms.fitness.service.internal.FitInternalBroker"
$ALL "com.google.android.gms/com.google.android.gms.fitness.service.proxy.FitProxyBroker"
$ALL "com.google.android.gms/com.google.android.gms.fitness.service.recording.FitRecordingBroker"
$ALL "com.google.android.gms/com.google.android.gms.fitness.service.sensors.FitSensorsBroker"
$ALL "com.google.android.gms/com.google.android.gms.fitness.service.sessions.FitSessionsBroker"
$ALL "com.google.android.gms/com.google.android.gms.fitness.sensors.sample.CollectSensorService"
$ALL "com.google.android.gms/com.google.android.gms.fitness.sync.FitnessSyncAdapterService" 
$ALL "com.google.android.gms/com.google.android.gms.fitness.sync.SyncGcmTaskService"

# Toggle services related to Google Nearby / Quick Share
$ALL "com.google.android.gms/com.google.android.gms.nearby.bootstrap.service.NearbyBootstrapService"
$ALL "com.google.android.gms/com.google.android.gms.nearby.connection.service.NearbyConnectionsAndroidService"
$ALL "com.google.android.gms/com.google.location.nearby.direct.service.NearbyDirectService"
$ALL "com.google.android.gms/com.google.android.gms.nearby.messages.service.NearbyMessagesService"

# Toggle services related to logging and data collection.
$ALL "com.google.android.gms/com.google.android.gms.analytics.internal.PlayLogReportingService"
$ALL "com.google.android.gms/com.google.android.gms.romanesco.ContactsLoggerUploadService"
$ALL "com.google.android.gms/com.google.android.gms.magictether.logging.DailyMetricsLoggerService"
$ALL "com.google.android.gms/com.google.android.gms.checkin.EventLogService"
$ALL "com.google.android.gms/com.google.android.gms.backup.component.FullBackupJobLoggerService"

# Toggle services related to security, app verification, and potentially network management.
$ALL "com.google.android.gms/com.google.android.gms.security.safebrowsing.SafeBrowsingUpdateTaskService"
$ALL "com.google.android.gms/com.google.android.gms.security.verifier.ApkUploadService"
$ALL "com.google.android.gms/com.google.android.gms.security.verifier.InternalApkUploadService"
$ALL "com.google.android.gms/com.google.android.gms.security.snet.SnetIdleTaskService"
$ALL "com.google.android.gms/com.google.android.gms.security.snet.SnetNormalTaskService"
$ALL "com.google.android.gms/com.google.android.gms.security.snet.SnetService"
$ALL "com.google.android.gms/com.google.android.gms.tapandpay.security.StorageKeyCacheService"
$ALL "com.google.android.gms/com.google.android.gms.droidguard.DroidGuardGcmTaskService"
$ALL "com.google.android.gms/com.google.android.gms.pay.security.storagekey.service.StorageKeyCacheService"

# Toggle services related to Google Pay (formerly Android Pay) & Google Wallet.
$ALL "com.google.android.gms/com.google.android.gms.tapandpay.gcmtask.TapAndPayGcmTaskService"
$ALL "com.google.android.gms/com.google.android.gms.tapandpay.globalactions.QuickAccessWalletService"
$ALL "com.google.android.gms/com.google.android.gms.tapandpay.globalactions.WalletQuickAccessWalletService"
$ALL "com.google.android.gms/com.google.android.gms.pay.gcmtask.PayGcmTaskService"
$ALL "com.google.android.gms/com.google.android.gms.pay.hce.service.PayHceService"
$ALL "com.google.android.gms/com.google.android.gms.pay.notifications.PayNotificationService"
$ALL "com.google.android.gms/com.google.android.gms.wallet.service.PaymentService"
$ALL "com.google.android.gms/com.google.android.gms.wallet.service.WalletGcmTaskService"

# Toggle services related to location.
$ALL "com.google.android.gms/com.google.android.gms.fitness.cache.DataUpdateListenerCacheService"
$ALL "com.google.android.gms/com.google.android.gms.fitness.sensors.sample.CollectSensorService"
$ALL "com.google.android.gms/com.google.android.gms.fitness.sync.SyncGcmTaskService"
$ALL "com.google.android.gms/com.google.android.location.fused.FusedLocationService"
$ALL "com.google.android.gms/com.google.android.location.internal.server.GoogleLocationService"
$ALL "com.google.android.gms/com.google.android.location.network.NetworkLocationService"
$ALL "com.google.android.gms/com.google.android.location.persistent.LocationPersistentService"
$ALL "com.google.android.gms/com.google.android.location.reporting.service.LocationHistoryInjectorService"
$ALL "com.google.android.gms/com.google.android.location.util.LocationAccuracyInjectorService"
$ALL "com.google.android.gms/com.google.android.location.wearable.WearableLocationService"
$ALL "com.google.android.gms/com.google.android.gms.locationsharing.service.LocationSharingSettingInjectorService"
$ALL "com.google.android.gms/com.google.android.gms.locationsharing.service.LocationSharingService"
$ALL "com.google.android.gms/com.google.android.gms.semanticlocation.service.SemanticLocationService"

# Toggle services related to components within Google Play Services related to Google Play Games.
$ALL "com.google.android.gms/com.google.android.gms.games.chimera.GamesSignInIntentServiceProxy"
$ALL "com.google.android.gms/com.google.android.gms.games.chimera.GamesSyncServiceNotificationProxy"
$ALL "com.google.android.gms/com.google.android.gms.games.chimera.GamesUploadServiceProxy"
$ALL "com.google.android.gms/com.google.android.gms.gp.gameservice.GameService"
$ALL "com.google.android.gms/com.google.android.gms.gp.gameservice.GameSessionService"

# Toggle services related to Google Instant Apps.
$ALL "com.google.android.gms/com.google.android.gms.chimera.GmsApiServiceNoInstantApps"
$ALL "com.google.android.gms/com.google.android.gms.chimera.PersistentApiServiceNoInstantApps"
$ALL "com.google.android.gms/com.google.android.gms.instantapps.service.InstantAppsService"
$ALL "com.google.android.gms/com.google.android.gms.chimera.UiApiServiceNoInstantApps"

## Safe Services (doesn't break anything)
# Google Play Instant Apps Services
pm disable "com.android.vending/com.google.android.fisnky.instantapps.InstantAppsLoggingService"
pm disable "com.google.android.gms/.instantapps.service.InstantAppsService"
pm disable "com.google.android.gms/com.google.android.finsky.instantapps.InstantAppsLoggingService"

# Google Work Apps Job Services
pm disable "com.google.android.apps.work.clouddpc/.base.policy.services.ReportingPartialCollectionJobService"
pm disable "com.google.android.apps.work.clouddpc/.base.policy.services.StatusReportJobService"
pm disable "com.google.android.apps.work.clouddpc/.vanilla.bugreport.jobs.RemoteBugReportJobService"

# Google Ads Services
pm disable "com.google.android.gms/.ads.AdRequestBrokerService"
pm disable "com.google.android.gms/.ads.cache.CacheBrokerService"
pm disable "com.google.android.gms/.ads.config.FlagsReceiver"
pm disable "com.google.android.gms/.ads.identifier.service.AdvertisingIdNotificationService"
pm disable "com.google.android.gms/.ads.identifier.service.AdvertisingIdService"
pm disable "com.google.android.gms/.ads.jams.NegotiationService"
pm disable "com.google.android.gms/.ads.measurement.GmpConversionTrackingBrokerService"
pm disable "com.google.android.gms/.ads.social.GcmSchedulerWakeupService"
pm disable "com.google.android.gms/.adsidentity.service.AdServicesExtDataStorageService"

# Google Analytics Services
pm disable "com.google.android.gms/.analytics.AnalyticsReceiver"
pm disable "com.google.android.gms/.analytics.AnalyticsService"
pm disable "com.google.android.gms/.analytics.AnalyticsTaskService"
pm disable "com.google.android.gms/.analytics.internal.PlayLogReportingService"
pm disable "com.google.android.gms/.analytics.service.AnalyticsService"

# Google Trust Agent Services
pm disable "com.google.android.gms/.auth.trustagent.ActiveUnlockTrustAgent"
pm disable "com.google.android.gms/.auth.trustagent.GoogleTrustAgent"

# Google Backup Services
pm disable "com.google.android.gms/.backup.component.FullBackupJobLoggerService"
pm disable "com.google.android.gms/.backup.stats.BackupStatsService"

# Google Check-in and Event Logging Services
pm disable "com.google.android.gms/.checkin.EventLogService"
pm disable "com.google.android.gms/.chimera.GmsIntentOperationService"
pm disable "com.google.android.gms/.chimera.container.logger.ExternalDebugLoggerService"
pm disable "com.google.android.gms/.common.appdoctor.LocalAppDoctorReceiver"
pm disable "com.google.android.gms/.common.stats.GmsCoreStatsService"
pm disable "com.google.android.gms/.common.stats.StatsUploadService"

# Google Feedback and Help Services
pm disable "com.google.android.gms/.feedback.LegacyBugReportService"
pm disable "com.google.android.gms/.feedback.OfflineReportSendTaskService"
pm disable "com.google.android.gms/.googlehelp.metrics.ReportBatchedMetricsGcmTaskService"

# Google Location Reporting Services
pm disable "com.google.android.gms/.location.reporting.service.GcmBroadcastReceiver"

# Magic Tether Services
pm disable "com.google.android.gms/.magictether.logging.DailyMetricsLoggerService"

# Google App Measurement Services
pm disable "com.google.android.gms/.measurement.AppMeasurementJobService"
pm disable "com.google.android.gms/.measurement.AppMeasurementReceiver"
pm disable "com.google.android.gms/.measurement.AppMeasurementService"
pm disable "com.google.android.gms/.measurement.PackageMeasurementReceiver"

# Google Contacts Logger Services
pm disable "com.google.android.gms/.romanesco.ContactsLoggerUploadService"

# Google Stats Services
pm disable "com.google.android.gms/.stats.PlatformStatsCollectorService"
pm disable "com.google.android.gms/.stats.eastworld.EastworldService"
pm disable "com.google.android.gms/.stats.service.DropBoxEntryAddedReceiver"
pm disable "com.google.android.gms/.stats.service.DropBoxEntryAddedService"

# Google Tron Services
pm disable "com.google.android.gms/.tron.AlarmReceiver"
pm disable "com.google.android.gms/.tron.CollectionService"

# Google UDC GCM Services
pm disable "com.google.android.gms/.udc.gcm.GcmBroadcastReceiver"

# Google Usage Reporting Services
pm disable "com.google.android.gms/.usagereporting.service.UsageReportingIntentService"

# Google MDM Services
pm disable "com.google.android.gms/com.google.android.gms.mdm.receivers.MdmDeviceAdminReceiver"
sleep 30
setprop mona-action rukho
) &

if [ ! "$(getprop mona-install)" == "rukho" ]; then
DEKHO "✦ Optimizing storage performance"
fstrim -v /cache || log "Error running fstrim on /cache: $?"
fstrim -v /system || log "Error running fstrim on /system: $?"
fstrim -v /vendor || log "Error running fstrim on /vendor: $?"
fstrim -v /data || log "Error running fstrim on /data: $?"
fstrim -v /persist || log "Error running fstrim on /persist: $?"
sleep 0.3
echo ""
DEKHO "✦ Deleting Logcat Buffer"
DEKHO "⌛ Please wait, This may take a while"
logcat -c > /dev/null 2>&1 || log "Error Nuking Logcat buffer: $?"
echo ""
dmesg -c > /dev/null 2>&1 || log "Error Nuking Dmesg logs: $?"

echo " "
echo " "
echo " "
echo "▄▀▀▀▄▄▄▄▄▄▄▀▀▀▄ "
echo "█▒▒░░░░░░░░░▒▒█ "
echo " █░ ░█░░░░░█░░█ "
echo "   █░░░▀█▀░░░█ "
echo "   ▀▄░░░░░░░▄▀ "
echo " "
echo "█▄█ █▀█ █▀█ █▀█ █▄█ "
echo "█▀█ █▀█ █▀▀ █▀▀ █"
sleep 1
echo " "
DEKHO "✦✦ ACTION COMPLETED SUCCESSFULY ✦✦" 3 "sar"
setprop mona-action khatam
exit 0
fi
EOF

# Make executable and Execute meow script
chmod +x "$MODPATH/action.sh"
chmod +x "$MODPATH/mona.sh"
. "$MODPATH/mona.sh"
ui_print ""
ui_print "⚠️ Note :- Magisk may crash after few seconds"
ui_print "  but Installation will get completed"
sleep 3
ui_print "- Finishing installation and redirecting "
ui_print "  to release source in"
sleep 0.3
ui_print "- 3s "
sleep 1
ui_print "- 2s "
sleep 1
ui_print "- 1s "
sleep 1

# Cleanup Hashes as it's not required after installation
find "$MODPATH" -type f -name "*.sha256" -exec rm -f {} \;
rm -rf "$MODPATH/verify.sh"

# Redirect Module Release Source and Finish Installation
am start -a android.intent.action.VIEW -d https://telegram.me/MeowRedirect/3 >/dev/null 2>&1
nohup sh -c "(
  sleep 6
  setprop mona-install khatam
)" &